@@ -3,6 +3,7 @@
 #endif
 
 #include <stdio.h>
+#include <string.h>
 
 #include <gtk/gtk.h>
 
@@ -11,13 +12,14 @@
 #include "support.h"
 
 
-/* Complicated dialog boxes can be slow to create, so we create them once and
-   keep pointers to them in static variables so we can reuse them. */
+/* We create the dialog boxes once and keep pointers to them in static
+   variables so we can reuse them. */
 static GtkWidget *open_filesel = NULL;
 static GtkWidget *save_filesel = NULL;
 static GtkWidget *fontsel = NULL;
 
-/* This is the filename of the file currently being edited. */
+/* This is the filename of the file currently being edited, in on-disk
+   encoding (which may be UTF-8 or the locale's encoding). */
 static gchar *current_filename = NULL;
 
 /* This flag is set to TRUE if the file has been changed. We don't actually
@@ -37,9 +39,29 @@ static void open_file (GtkWidget *main_window);
 static void save_as (GtkWidget *main_window);
 
 static void real_open_file (GtkWidget *main_window,
-			    gchar *filename);
+			    const gchar *filename);
 static void real_save_file (GtkWidget *main_window,
-			    gchar *filename);
+			    const gchar *filename);
+
+
+/***************************************************************************
+ * Main Window Signals.
+ ***************************************************************************/
+
+/* This is called when the main window gets a "delete_event" signal, which
+   typically happens when the user clicks on the close icon on the window's
+   title bar. If we didn't handle this, the window would be destroyed but
+   our application would not exit. */
+gboolean
+on_main_window_delete_event            (GtkWidget       *widget,
+                                        GdkEvent        *event,
+                                        gpointer         user_data)
+{
+  gtk_exit (0);
+
+  /* Shouldn't reach here, but we add it to stop compiler warnings. */
+  return FALSE;
+}
 
 
 /***************************************************************************
@@ -108,9 +130,10 @@ on_Exit_activate                       (GtkMenuItem     *menuitem,
 
 /***************************************************************************
  * Edit Menu Commands.
- * The GtkText superclass GtkEditable makes this very easy for us by
- * providing clipboard functions.
+ * The GtkTextView class makes this very easy for us by providing action
+ * signals that perform clipboard functions.
  ***************************************************************************/
+
 void
 on_Cut_activate                        (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
@@ -118,7 +141,7 @@ on_Cut_activate                        (GtkMenuItem     *menuitem,
   GtkWidget *text, *statusbar;
 
   text = lookup_widget (GTK_WIDGET (menuitem), "text1");
-  gtk_editable_cut_clipboard (GTK_EDITABLE (text));
+  g_signal_emit_by_name (G_OBJECT (text), "cut_clipboard", NULL);
 
   statusbar = lookup_widget (GTK_WIDGET (menuitem), "statusbar1");
   gtk_statusbar_pop (GTK_STATUSBAR (statusbar), 1);
@@ -133,7 +156,7 @@ on_Copy_activate                       (GtkMenuItem     *menuitem,
   GtkWidget *text, *statusbar;
 
   text = lookup_widget (GTK_WIDGET (menuitem), "text1");
-  gtk_editable_copy_clipboard (GTK_EDITABLE (text));
+  g_signal_emit_by_name (G_OBJECT (text), "copy_clipboard", NULL);
 
   statusbar = lookup_widget (GTK_WIDGET (menuitem), "statusbar1");
   gtk_statusbar_pop (GTK_STATUSBAR (statusbar), 1);
@@ -148,7 +171,7 @@ on_Paste_activate                      (GtkMenuItem     *menuitem,
   GtkWidget *text, *statusbar;
 
   text = lookup_widget (GTK_WIDGET (menuitem), "text1");
-  gtk_editable_paste_clipboard (GTK_EDITABLE (text));
+  g_signal_emit_by_name (G_OBJECT (text), "paste_clipboard", NULL);
 
   statusbar = lookup_widget (GTK_WIDGET (menuitem), "statusbar1");
   gtk_statusbar_pop (GTK_STATUSBAR (statusbar), 1);
@@ -157,13 +180,15 @@ on_Paste_activate                      (GtkMenuItem     *menuitem,
 
 
 void
-on_Clear_activate                     (GtkMenuItem     *menuitem,
-				       gpointer         user_data)
+on_Clear_activate                      (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
 {
   GtkWidget *text, *statusbar;
+  GtkTextBuffer *buffer;
 
   text = lookup_widget (GTK_WIDGET (menuitem), "text1");
-  gtk_editable_delete_selection (GTK_EDITABLE (text));
+  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text));
+  gtk_text_buffer_delete_selection (buffer, TRUE, TRUE);
 
   statusbar = lookup_widget (GTK_WIDGET (menuitem), "statusbar1");
   gtk_statusbar_pop (GTK_STATUSBAR (statusbar), 1);
@@ -174,6 +199,7 @@ on_Clear_activate                     (GtkMenuItem     *menuitem,
 /***************************************************************************
  * Toolbar Buttons.
  ***************************************************************************/
+
 void
 on_new_button_clicked                  (GtkButton       *button,
                                         gpointer         user_data)
@@ -211,26 +237,6 @@ on_save_button_clicked                 (GtkButton       *button,
 }
 
 
-/***************************************************************************
- * Main Window Signals.
- ***************************************************************************/
-
-/* This is called when the main window gets a "delete_event" signal, which
-   typically happens when the user clicks on the close icon on the window's
-   title bar. If we didn't handle this, the window would be destroyed but
-   our application would not exit. */
-gboolean
-on_main_window_delete_event            (GtkWidget       *widget,
-                                        GdkEvent        *event,
-                                        gpointer         user_data)
-{
-  gtk_exit (0);
-
-  /* Shouldn't reach here, but we add it to stop compiler warnings. */
-  return FALSE;
-}
-
-
 /***************************************************************************
  * Font Selection Dialog.
  ***************************************************************************/
@@ -248,42 +254,25 @@ on_Font_activate                       (GtkMenuItem     *menuitem,
     {
       fontsel = create_font_selection ();
       gtk_object_set_data (GTK_OBJECT (fontsel), MainWindowKey, main_window);
+
+      /* Make sure the dialog doesn't disappear behind the main window. */
+      gtk_window_set_transient_for (GTK_WINDOW (fontsel), 
+				    GTK_WINDOW (main_window));
     }
   gtk_widget_show (fontsel);
 
-  /* We raise the window in case it is already created and shown, but is
-     hidden behind another window. */
-  gdk_window_raise (fontsel->window);
+  /* We make sure the dialog is visible. */
+  gtk_window_present (GTK_WINDOW (fontsel));
 }
 
 
-void
-on_fontsel_apply_button_clicked        (GtkButton       *button,
+gboolean
+on_fontsel_delete_event                (GtkWidget       *widget,
+                                        GdkEvent        *event,
                                         gpointer         user_data)
 {
-  GtkWidget *fontsel, *main_window, *text, *statusbar;
-  GtkStyle *style;
-  GdkFont *font;
-
-  fontsel = gtk_widget_get_toplevel (GTK_WIDGET (button));
-  main_window = gtk_object_get_data (GTK_OBJECT (fontsel), MainWindowKey);
-  text = lookup_widget (main_window, "text1");
-  statusbar = lookup_widget (main_window, "statusbar1");
-  font = gtk_font_selection_dialog_get_font (GTK_FONT_SELECTION_DIALOG (fontsel));
-
-  /* If no font is selected, or the font couldn't be loaded, just return. */
-  if (!font)
-    return;
-
-  /* We copy the current style, and update the font. */
-  style = gtk_style_copy (text->style);
-  gdk_font_unref (style->font);
-  style->font = font;
-  gdk_font_ref (font);
-  gtk_widget_set_style (text, style);
-
-  gtk_statusbar_pop (GTK_STATUSBAR (statusbar), 1);
-  gtk_statusbar_push (GTK_STATUSBAR (statusbar), 1, "Font updated.");
+  gtk_widget_hide (gtk_widget_get_toplevel (widget));
+  return TRUE;
 }
 
 
@@ -308,42 +297,42 @@ on_fontsel_cancel_button_clicked       (GtkButton       *button,
 }
 
 
-gboolean
-on_fontsel_delete_event                (GtkWidget       *widget,
-                                        GdkEvent        *event,
-                                        gpointer         user_data)
-{
-  gtk_widget_hide (gtk_widget_get_toplevel (widget));
-  return TRUE;
-}
-
-
-/***************************************************************************
- * Text Widget signals.
- ***************************************************************************/
-
-/*
- * Text changed callback. This signal is emitted whenever the text in the
- * GtkText changes. We don't use this at present, but it could be used to
- * prompt to save changes before opening new files or quitting.
- */
 void
-on_text_changed                        (GtkEditable     *editable,
+on_fontsel_apply_button_clicked        (GtkButton       *button,
                                         gpointer         user_data)
 {
-  file_changed = TRUE;
+  GtkWidget *fontsel, *main_window, *text, *statusbar;
+  gchar *font_name;
+  PangoFontDescription *font_desc;
+
+  fontsel = gtk_widget_get_toplevel (GTK_WIDGET (button));
+  main_window = gtk_object_get_data (GTK_OBJECT (fontsel), MainWindowKey);
+  text = lookup_widget (main_window, "text1");
+  statusbar = lookup_widget (main_window, "statusbar1");
+
+  font_name = gtk_font_selection_dialog_get_font_name (GTK_FONT_SELECTION_DIALOG (fontsel));
+  font_desc = pango_font_description_from_string (font_name);
+  g_free (font_name);
+
+  /* We copy the current style, and update the font. */
+  gtk_widget_modify_font (text, font_desc);
+  pango_font_description_free (font_desc);
+
+  gtk_statusbar_pop (GTK_STATUSBAR (statusbar), 1);
+  gtk_statusbar_push (GTK_STATUSBAR (statusbar), 1, "Font updated.");
 }
 
 
 /***************************************************************************
  * Open File Selection Dialog.
  ***************************************************************************/
+
 void
-on_open_filesel_ok_button_clicked           (GtkButton       *button,
-					     gpointer         user_data)
+on_open_filesel_ok_button_clicked      (GtkButton       *button,
+                                        gpointer         user_data)
 {
   GtkWidget *filesel, *main_window;
-  gchar *filename;
+  const gchar *filename;
 
   filesel = gtk_widget_get_toplevel (GTK_WIDGET (button));
   main_window = gtk_object_get_data (GTK_OBJECT (filesel), MainWindowKey);
@@ -354,8 +343,8 @@ on_open_filesel_ok_button_clicked           (GtkButton       *button,
 
 
 void
-on_open_filesel_cancel_button_clicked       (GtkButton       *button,
-					     gpointer         user_data)
+on_open_filesel_cancel_button_clicked  (GtkButton       *button,
+                                        gpointer         user_data)
 {
   gtk_widget_hide (gtk_widget_get_toplevel (GTK_WIDGET (button)));
 }
@@ -374,12 +363,13 @@ on_open_filesel_delete_event           (GtkWidget       *widget,
 /***************************************************************************
  * Save File Selection Dialog.
  ***************************************************************************/
+
 void
-on_save_filesel_ok_button_clicked           (GtkButton       *button,
-					     gpointer         user_data)
+on_save_filesel_ok_button_clicked      (GtkButton       *button,
+                                        gpointer         user_data)
 {
   GtkWidget *filesel, *main_window;
-  gchar *filename;
+  const gchar *filename;
 
   filesel = gtk_widget_get_toplevel (GTK_WIDGET (button));
   main_window = gtk_object_get_data (GTK_OBJECT (filesel), MainWindowKey);
@@ -390,8 +380,8 @@ on_save_filesel_ok_button_clicked           (GtkButton       *button,
 
 
 void
-on_save_filesel_cancel_button_clicked       (GtkButton       *button,
-					     gpointer         user_data)
+on_save_filesel_cancel_button_clicked  (GtkButton       *button,
+                                        gpointer         user_data)
 {
   gtk_widget_hide (gtk_widget_get_toplevel (GTK_WIDGET (button)));
 }
@@ -410,13 +400,20 @@ on_save_filesel_delete_event           (GtkWidget       *widget,
 /***************************************************************************
  * About Dialog.
  ***************************************************************************/
+
 void
 on_About_activate                      (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
   GtkWidget *about;
+  GtkWidget *main_window;
+
+  main_window = lookup_widget (GTK_WIDGET (menuitem), "main_window");
 
   about = create_about_dialog ();
+  /* Make sure the dialog doesn't disappear behind the main window. */
+  gtk_window_set_transient_for (GTK_WINDOW (about), 
+				GTK_WINDOW (main_window));
   gtk_widget_show (about);
 }
 
@@ -429,6 +426,22 @@ on_about_ok_clicked                    (GtkButton       *button,
 }
 
 
+/***************************************************************************
+ * Text Widget signals.
+ ***************************************************************************/
+
+/*
+ * Text changed callback. This signal is emitted whenever the text in the
+ * GtkTextView changes. We don't use this at present, but it could be used to
+ * prompt to save changes before opening new files or quitting.
+ */
+void
+on_text_changed                        (GtkTextBuffer   *buffer,
+                                        gpointer         user_data)
+{
+  file_changed = TRUE;
+}
+
 
 /***************************************************************************
  * Private functions.
@@ -438,14 +451,20 @@ on_about_ok_clicked                    (GtkButton       *button,
 void
 set_window_title (GtkWidget *main_window)
 {
-  gchar *title;
+  gchar *title, *filename_utf8 = NULL;
+
+  /* We need to convert the filename to UTF-8. */
+  if (current_filename)
+    filename_utf8 = g_filename_to_utf8 (current_filename, -1, NULL, NULL,
+					NULL);
 
   title = g_strdup_printf ("Editor: %s\n",
-			   current_filename ? g_basename (current_filename)
-			                    : "untitled");
+			   filename_utf8 ? g_basename (filename_utf8)
+			   : "untitled");
 
   gtk_window_set_title (GTK_WINDOW (main_window), title);
   g_free (title);
+  g_free (filename_utf8);
 }
 
 
@@ -455,9 +474,14 @@ static void
 new_file (GtkWidget *main_window)
 {
   GtkWidget *text, *statusbar;
+  GtkTextBuffer *buffer;
+  GtkTextIter start, end;
 
   text = lookup_widget (GTK_WIDGET (main_window), "text1");
-  gtk_editable_delete_text (GTK_EDITABLE (text), 0, -1);
+  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text));
+  gtk_text_buffer_get_bounds (buffer, &start, &end);
+  gtk_text_buffer_delete (buffer, &start, &end);
+
   g_free (current_filename);
   current_filename = NULL;
   file_changed = FALSE;
@@ -476,14 +500,19 @@ open_file (GtkWidget *main_window)
   /* We use the same file selection widget each time, so first
      of all we create it if it hasn't already been created. */
   if (open_filesel == NULL)
-    open_filesel = create_open_file_selection ();
+    {
+      open_filesel = create_open_file_selection ();
+      /* Make sure the dialog doesn't disappear behind the main window. */
+      gtk_window_set_transient_for (GTK_WINDOW (open_filesel), 
+				    GTK_WINDOW (main_window));
+    }
 
   /* We save a pointer to the main window inside the file selection's
      data list, so we can get it easily in the callbacks. */
   gtk_object_set_data (GTK_OBJECT (open_filesel), MainWindowKey, main_window);
 
-  gtk_widget_show (open_filesel);
-  gdk_window_raise (open_filesel->window);
+  /* We make sure the dialog is visible. */
+  gtk_window_present (GTK_WINDOW (open_filesel));
 }
 
 
@@ -492,7 +521,12 @@ static void
 save_as (GtkWidget *main_window)
 {
   if (save_filesel == NULL)
-    save_filesel = create_save_file_selection ();
+    {
+      save_filesel = create_save_file_selection ();
+      /* Make sure the dialog doesn't disappear behind the main window. */
+      gtk_window_set_transient_for (GTK_WINDOW (save_filesel), 
+				    GTK_WINDOW (main_window));
+    }
 
   gtk_object_set_data (GTK_OBJECT (save_filesel), MainWindowKey, main_window);
 
@@ -501,25 +535,29 @@ save_as (GtkWidget *main_window)
     gtk_file_selection_set_filename (GTK_FILE_SELECTION (save_filesel),
 				     current_filename);
 
-  gtk_widget_show (save_filesel);
-  gdk_window_raise (save_filesel->window);
+  /* We make sure the dialog is visible. */
+  gtk_window_present (GTK_WINDOW (save_filesel));
 }
 
 
-/* This loads the given file. */
+/* This loads the given file, which is in on-disk encoding (which may not
+   be UTF-8). */
 static void
-real_open_file (GtkWidget *main_window, gchar *filename)
+real_open_file (GtkWidget *main_window, const gchar *filename)
 {
   GtkWidget *text, *statusbar;
+  GtkTextBuffer *text_buffer;
+  GtkTextIter start, end;
   FILE *fp;
   gchar buffer[BUFFER_SIZE];
   gint bytes_read;
 
   text = lookup_widget (main_window, "text1");
+  text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text));
   statusbar = lookup_widget (main_window, "statusbar1");
   gtk_statusbar_pop (GTK_STATUSBAR (statusbar), 1);
-  gtk_text_freeze (GTK_TEXT (text));
-  gtk_editable_delete_text (GTK_EDITABLE (text), 0, -1);
+  gtk_text_buffer_get_bounds (text_buffer, &start, &end);
+  gtk_text_buffer_delete (text_buffer, &start, &end);
   g_free (current_filename);
   current_filename = NULL;
   file_changed = FALSE;
@@ -527,7 +565,6 @@ real_open_file (GtkWidget *main_window, gchar *filename)
   fp = fopen (filename, "r");
   if (fp == NULL)
     {
-      gtk_text_thaw (GTK_TEXT (text));
       gtk_statusbar_push (GTK_STATUSBAR (statusbar), 1, "Couldn't open file.");
       return;
     }
@@ -537,8 +574,10 @@ real_open_file (GtkWidget *main_window, gchar *filename)
       bytes_read = fread (buffer, sizeof (gchar), BUFFER_SIZE, fp);
 
       if (bytes_read > 0)
-	gtk_text_insert (GTK_TEXT (text), NULL, NULL, NULL, buffer,
-			 bytes_read);
+	{
+	  gtk_text_buffer_get_end_iter (text_buffer, &end);
+	  gtk_text_buffer_insert (text_buffer, &end, buffer, bytes_read);
+	}
 
       if (bytes_read != BUFFER_SIZE && (feof (fp) || ferror (fp)))
 	break;
@@ -549,15 +588,14 @@ real_open_file (GtkWidget *main_window, gchar *filename)
   if (ferror (fp))
     {
       fclose (fp);
-      gtk_editable_delete_text (GTK_EDITABLE (text), 0, -1);
-      gtk_text_thaw (GTK_TEXT (text));
+      gtk_text_buffer_get_bounds (text_buffer, &start, &end);
+      gtk_text_buffer_delete (text_buffer, &start, &end);
       set_window_title (main_window);
       gtk_statusbar_push (GTK_STATUSBAR (statusbar), 1, "Error loading file.");
       return;
     }
 
   fclose (fp);
-  gtk_text_thaw (GTK_TEXT (text));
 
   current_filename = g_strdup (filename);
   set_window_title (main_window);
@@ -566,10 +604,14 @@ real_open_file (GtkWidget *main_window, gchar *filename)
 }
 
 
+/* This saves the file, which is in on-disk encoding (which may not
+   be UTF-8). */
 static void
-real_save_file (GtkWidget *main_window, gchar *filename)
+real_save_file (GtkWidget *main_window, const gchar *filename)
 {
   GtkWidget *text, *statusbar;
+  GtkTextBuffer *text_buffer;
+  GtkTextIter start, end;
   gchar *data;
   FILE *fp;
   gint bytes_written, len;
@@ -582,8 +624,11 @@ real_save_file (GtkWidget *main_window, gchar *filename)
     }
 
   text = lookup_widget (GTK_WIDGET (main_window), "text1");
+  text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text));
   statusbar = lookup_widget (main_window, "statusbar1");
-  data = gtk_editable_get_chars (GTK_EDITABLE (text), 0, -1);
+
+  gtk_text_buffer_get_bounds (text_buffer, &start, &end);
+  data = gtk_text_buffer_get_text (text_buffer, &start, &end, TRUE);
 
   fp = fopen (filename, "w");
   if (fp == NULL)
