@@ -566,7 +566,14 @@ gb_widget_write_signal_handler_source (GtkWidget *widget,
 
 
   signal_id = gtk_signal_lookup (signal_name, GTK_OBJECT_TYPE (widget));
-  g_return_if_fail (signal_id != 0);
+  /* If a project is converted from GTK+ 1.2.x to 2.x.x it may have some
+     signals which aren't valid any more. So we output a warning. */
+  if (signal_id == 0)
+    {
+      g_warning ("Invalid signal: '%s' for widget: '%s'", signal_name,
+		 data->wname);
+      return;
+    }
 
   g_signal_query (signal_id, &query_info);
 
