@@ -87,8 +87,6 @@ static void on_clist_dialog_destroy (GtkWidget * widget,
 static GtkWidget *new_label (GtkWidget * parent);
 static GtkWidget *new_unnamed_label (GtkWidget * parent);
 
-static void gb_clist_adjustment_changed (GtkAdjustment *adjustment,
-					 GtkWidget     *widget);
 
 /******
  * NOTE: To use these functions you need to uncomment them AND add a pointer
@@ -235,19 +233,6 @@ new_unnamed_label (GtkWidget * parent)
 }
 
 
-/* This just redraws the entire widget when it is scrolled, to make sure that
-   it isn't messed up due to our selection handles. */
-static void
-gb_clist_adjustment_changed (GtkAdjustment *adjustment,
-			     GtkWidget     *widget)
-{
-  /* We check that this is a widget in the interface being created rather
-     than part of Glade's interface. */
-  if (GB_IS_GB_WIDGET (widget))
-    gtk_widget_queue_clear (widget);
-}
-
-
 /*
  * Creates the components needed to edit the extra properties of this widget.
  */
@@ -413,6 +398,11 @@ gb_clist_add_child (GtkWidget *widget, GtkWidget *child, GbWidgetSetArgData *dat
   gint col = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (widget),
 						   "last_child"));
 
+  if (col >= GTK_CLIST (widget)->columns) {
+    g_warning ("Too many column title widgets for GtkCList - skipping");
+    return;
+  }
+
   gtk_clist_set_column_widget (GTK_CLIST (widget), col, child);
   gtk_object_set_data (GTK_OBJECT (widget), "last_child",
 		       GINT_TO_POINTER (col + 1));
@@ -522,17 +512,29 @@ gb_clist_write_add_child_source (GtkWidget * parent,
 
 
 
-static gboolean
-gb_clist_emission_hook (GSignalInvocationHint  *ihint,
-			guint			n_param_values,
-			const GValue	       *param_values,
-			gpointer		data)
+/* This just redraws the entire widget when it is scrolled, to make sure that
+   it isn't messed up due to our selection handles. */
+static void
+gb_scroll_adjustment_changed (GtkAdjustment *adjustment,
+			     GtkWidget     *widget)
+{
+  /* We check that this is a widget in the interface being created rather
+     than part of Glade's interface. */
+  if (GB_IS_GB_WIDGET (widget))
+    gtk_widget_queue_clear (widget);
+}
+
+
+gboolean
+gb_set_scroll_adjustments_hook (GSignalInvocationHint  *ihint,
+				guint			n_param_values,
+				const GValue	       *param_values,
+				gpointer		data)
 {
   GtkObject *object, *hadjustment, *vadjustment;
   GtkObject *old_hadjustment, *old_vadjustment;
 
   object = g_value_get_object (param_values);
-  g_return_val_if_fail (GTK_IS_CLIST (object), FALSE);
 
   hadjustment = g_value_get_object (param_values + 1);
   vadjustment = g_value_get_object (param_values + 2);
@@ -544,7 +546,7 @@ gb_clist_emission_hook (GSignalInvocationHint  *ihint,
 
       if (hadjustment)
 	gtk_signal_connect (hadjustment, "value_changed",
-			    (GtkSignalFunc) gb_clist_adjustment_changed,
+			    (GtkSignalFunc) gb_scroll_adjustment_changed,
 			    object);
     }
 
@@ -555,7 +557,7 @@ gb_clist_emission_hook (GSignalInvocationHint  *ihint,
 
       if (vadjustment)
 	gtk_signal_connect (vadjustment, "value_changed",
-			    (GtkSignalFunc) gb_clist_adjustment_changed,
+			    (GtkSignalFunc) gb_scroll_adjustment_changed,
 			    object);
     }
 
@@ -582,7 +584,7 @@ gb_clist_init ()
      work for subclasses of GtkCList. */
   klass = gtk_type_class (gtk_clist_get_type ());
   g_signal_add_emission_hook (klass->set_scroll_adjustments_signal, 0,
-			      gb_clist_emission_hook, NULL, NULL);
+			      gb_set_scroll_adjustments_hook, NULL, NULL);
 
   /* Initialize the GbWidget structure */
   gb_widget_init_struct (&gbwidget);
