@@ -298,7 +298,7 @@ show_dialog_creation_dialog (GbWidgetNewData * data)
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
   gtk_radio_button_set_group (GTK_RADIO_BUTTON (cancel_ok), layout_group_group);
-  layout_group_group = gtk_radio_button_group (GTK_RADIO_BUTTON (cancel_ok));
+  layout_group_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (cancel_ok));
 
   just_ok = gtk_radio_button_new_with_mnemonic (NULL, _("OK"));
   gtk_object_set_data (GTK_OBJECT (dialog), "just_ok", just_ok);
@@ -307,7 +307,7 @@ show_dialog_creation_dialog (GbWidgetNewData * data)
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
   gtk_radio_button_set_group (GTK_RADIO_BUTTON (just_ok), layout_group_group);
-  layout_group_group = gtk_radio_button_group (GTK_RADIO_BUTTON (just_ok));
+  layout_group_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (just_ok));
 
   cancel_apply_ok = gtk_radio_button_new_with_mnemonic (NULL, _("Cancel, Apply, OK"));
   gtk_object_set_data (GTK_OBJECT (dialog), "cancel_apply_ok", cancel_apply_ok);
@@ -316,7 +316,7 @@ show_dialog_creation_dialog (GbWidgetNewData * data)
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
   gtk_radio_button_set_group (GTK_RADIO_BUTTON (cancel_apply_ok), layout_group_group);
-  layout_group_group = gtk_radio_button_group (GTK_RADIO_BUTTON (cancel_apply_ok));
+  layout_group_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (cancel_apply_ok));
 
   just_close = gtk_radio_button_new_with_mnemonic (NULL, _("Close"));
   gtk_object_set_data (GTK_OBJECT (dialog), "just_close", just_close);
@@ -325,7 +325,7 @@ show_dialog_creation_dialog (GbWidgetNewData * data)
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
   gtk_radio_button_set_group (GTK_RADIO_BUTTON (just_close), layout_group_group);
-  layout_group_group = gtk_radio_button_group (GTK_RADIO_BUTTON (just_close));
+  layout_group_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (just_close));
 
   standard_layout = gtk_radio_button_new_with_mnemonic (NULL, _("_Standard Button Layout:"));
   gtk_object_set_data (GTK_OBJECT (dialog), "standard_layout", standard_layout);
@@ -334,7 +334,7 @@ show_dialog_creation_dialog (GbWidgetNewData * data)
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
   gtk_radio_button_set_group (GTK_RADIO_BUTTON (standard_layout), main_group_group);
-  main_group_group = gtk_radio_button_group (GTK_RADIO_BUTTON (standard_layout));
+  main_group_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (standard_layout));
 
   number_of_buttons = gtk_radio_button_new_with_mnemonic (NULL, _("_Number of Buttons:"));
   gtk_widget_show (number_of_buttons);
@@ -342,7 +342,7 @@ show_dialog_creation_dialog (GbWidgetNewData * data)
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (0), 0, 0);
   gtk_radio_button_set_group (GTK_RADIO_BUTTON (number_of_buttons), main_group_group);
-  main_group_group = gtk_radio_button_group (GTK_RADIO_BUTTON (number_of_buttons));
+  main_group_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (number_of_buttons));
 
   num_buttons_adj = gtk_adjustment_new (1, 0, 100, 1, 10, 10);
   num_buttons = gtk_spin_button_new (GTK_ADJUSTMENT (num_buttons_adj), 1, 0);
