@@ -129,24 +129,25 @@ check_visible_sizes (const gchar *stock_id)
   gboolean item_visible[G_N_ELEMENTS(GladeIconSizeValues)];
   GList *children;
 
-  icon_set = gtk_icon_factory_lookup_default (stock_id);
-  gtk_icon_set_get_sizes (icon_set, &sizes, &n_sizes);
-
   for (j = 0; j < GladeIconSizeChoicesSize; j++)
     item_visible[j] = FALSE;
 
-  /* Figure out which of our choices should be visible. */
-  for (i = 0; i < n_sizes; i++)
+  icon_set = gtk_icon_factory_lookup_default (stock_id);
+  if (icon_set)
     {
-      for (j = 0; j < GladeIconSizeChoicesSize; j++)
+      gtk_icon_set_get_sizes (icon_set, &sizes, &n_sizes);
+      /* Figure out which of our choices should be visible. */
+      for (i = 0; i < n_sizes; i++)
 	{
-	  if (sizes[i] == GladeIconSizeValues[j])
-	    item_visible[j] = TRUE;
+	  for (j = 0; j < GladeIconSizeChoicesSize; j++)
+	    {
+	      if (sizes[i] == GladeIconSizeValues[j])
+		item_visible[j] = TRUE;
+	    }
 	}
+      g_free (sizes);
     }
 
-  g_free (sizes);
-
   /* Show or Hide the items as appropriate. */
   option_menu = property_get_value_widget (IconSize);
   g_return_if_fail (GTK_IS_OPTION_MENU (option_menu));
@@ -236,20 +237,24 @@ static GtkIconSize
 check_icon_size (const gchar *stock_id, GtkIconSize icon_size)
 {
   GtkIconSet *icon_set;
-  GtkIconSize *sizes, retval;
+  GtkIconSize *sizes, retval = GTK_ICON_SIZE_BUTTON;
   gint n_sizes, i;
 
   icon_set = gtk_icon_factory_lookup_default (stock_id);
-  gtk_icon_set_get_sizes (icon_set, &sizes, &n_sizes);
-
-  for (i = 0; i < n_sizes; i++)
+  if (icon_set)
     {
-      if (sizes[i] == icon_size)
-	return icon_size;
+      gtk_icon_set_get_sizes (icon_set, &sizes, &n_sizes);
+
+      for (i = 0; i < n_sizes; i++)
+	{
+	  if (sizes[i] == icon_size)
+	    return icon_size;
+	}
+
+      retval = sizes[0];
+      g_free (sizes);
     }
 
-  retval = sizes[0];
-  g_free (sizes);
   return retval;
 }
 
