@@ -129,10 +129,13 @@ gb_menu_write_source (GtkWidget * widget, GbWidgetWriteSourceData * data)
      the menu from the GnomeUIInfo structs. */
   if (data->project->gnome_support)
     {
+      GtkWidget *attach_widget;
+
       glade_gnome_start_menu_source (GTK_MENU_SHELL (widget), data);
 
       /* We only need to create the toplevel menu. */
-      if (gtk_menu_get_attach_widget (GTK_MENU (widget)) == NULL)
+      attach_widget = gtk_menu_get_attach_widget (GTK_MENU (widget));
+      if (attach_widget == NULL || GTK_IS_OPTION_MENU (attach_widget))
 	{
 	  if (data->create_widget)
 	    {
