@@ -140,7 +140,7 @@ gb_toolbar_new(GbWidgetNewData *data)
 void
 gb_toolbar_add_child (GtkWidget *widget, GtkWidget *child, GbWidgetSetArgData *data)
 {
-  gchar *class_name;
+  gchar *class_name = NULL;
   gboolean new_group;
 
   /* Tell the load functions to use the child properties array. */
@@ -647,7 +647,7 @@ gb_toolbar_new_toolbar_button (GbWidgetNewData *data,
   MSG ("In gb_widget_new_toolbar_button");
   if (data->action == GB_LOADING)
     {
-      gchar *class_name;
+      gchar *class_name = NULL;
 
       if (data->loading_data->widget_info)
 	class_name = data->loading_data->widget_info->class;
@@ -802,7 +802,7 @@ gb_toolbar_write_source (GtkWidget * widget, GbWidgetWriteSourceData * data)
 
   if (GTK_TOOLBAR (widget)->orientation != GTK_ORIENTATION_HORIZONTAL)
     source_add (data,
-		"  gtk_toolbar_set_style (GTK_TOOLBAR (%s), %s);\n",
+		"  gtk_toolbar_set_orientation (GTK_TOOLBAR (%s), %s);\n",
 		data->wname, orientation);
 
   if (!GTK_TOOLTIPS (GTK_TOOLBAR (widget)->tooltips)->enabled)
