@@ -41,7 +41,6 @@ GbWidget *gb_gnome_db_list_init ();
 GbWidget *gb_gnome_db_login_init ();
 GbWidget *gb_gnome_db_logindlg_init ();
 GbWidget *gb_gnome_db_provider_selector_init ();
-GbWidget *gb_gnome_db_sql_editor_init ();
 GbWidget *gb_gnome_db_table_editor_init ();
 
 static GladeWidgetInitData gnome_db[] = {
@@ -55,10 +54,10 @@ static GladeWidgetInitData gnome_db[] = {
 	{ "GnomeDbProviderSelector",	gb_gnome_db_provider_selector_init },
 	{ "GnomeDbConnectionSelector",	gb_gnome_db_connection_selector_init },
 
+	{ "GnomeDbDataSourceSelector",	gb_gnome_db_data_source_selector_init },
         { "GnomeDbBrowser",     gb_gnome_db_browser_init },
 	{ "GnomeDbTableEditor",	gb_gnome_db_table_editor_init },
 	{ "GnomeDbForm",	gb_gnome_db_form_init },
-	{ "GnomeDbSqlEditor",	gb_gnome_db_sql_editor_init },
 
         { "GnomeDbGrid",        gb_gnome_db_grid_init },
         { "GnomeDbList",        gb_gnome_db_list_init },
@@ -75,17 +74,14 @@ static GladeWidgetInitData gnome_db[] = {
 
 static GladeWidgetInitData notshown[] =
 {
-  /* This is useless at present (it needs to have a provider set when
-     created, and we can't do that), so we don't show it. */
-  { "GnomeDbDataSourceSelector",	gb_gnome_db_data_source_selector_init },
-
-  /* Unfinished. */
   { NULL, NULL }
 };
 
 
 static GladePaletteSectionData sections[] =
 {
+  /* Note that glade_palette_set_show_gnome_widgets() has some of these
+     strings hard-coded now, so keep up-to-date. */
   { "Gnome _DB", gnome_db },
   { "NotShown", notshown },
   { NULL, NULL }
