@@ -26,7 +26,7 @@
 #include <libxml/parser.h>
 
 #include <gtk/gtkmenu.h>
-
+#include <gtk/gtk.h>
 #include "gladeconfig.h"
 
 #include "editor.h"
@@ -443,14 +443,69 @@ glade_project_write_c_source (GladeProject *project)
   return source_write (project);
 }
 
+/*
+ * Iterate through the list of widgets to ensure we're not using any
+ * deprecated widgets in the project when emitting C++ code.
+ */
+static gboolean
+check_deprecated_widget (GList* list)
+{
+    GtkWidget* widget;
+    GList* tmpList;
+    gboolean status;
+
+    for (; list != NULL; list = list->next )
+      {
+        widget = GTK_WIDGET (list->data);
+
+        if (GTK_IS_CONTAINER (widget))
+	  {
+            tmpList = gtk_container_get_children (GTK_CONTAINER (widget));
+            status = check_deprecated_widget (tmpList);
+            g_list_free (tmpList);
+
+            if (!status)
+	      {
+                return FALSE;
+	      }
+	  }
+
+	/* Check if any deprecated widgets are being used. Note that we only
+	   check for widgets created by Glade, since GtkCombo uses a GtkList
+	   internally and we don't want to disallow that. */
+        if (GB_IS_GB_WIDGET (widget)
+	    && (GTK_IS_CLIST        (widget) ||
+		GTK_IS_CTREE        (widget) ||
+		GTK_IS_LIST         (widget) ||
+		GTK_IS_PIXMAP       (widget) ||
+		GTK_IS_PREVIEW      (widget) ))
+	  {
+            return FALSE;
+	  }
+      }
+
+    return TRUE;
+}
 
 /* Use system() to run glade-- on the XML file to generate C++ source code. */
 static GladeError*
 glade_project_write_cxx_source (GladeProject *project)
 {
-  gchar *command_buffer;
-  gint status;
- 
+    gchar *command_buffer;
+    gint status;
+
+    /*
+     * Ensure that our project does not have deprecated widgets
+     * that gtkmm-2 doesn't support.
+     */
+    if ( !check_deprecated_widget (project->components))
+      {
+        return glade_error_new_general (GLADE_STATUS_ERROR,
+		_("Your project uses deprecated widgets that Gtkmm-2\n"
+		  "does not support.  Check your project for these\n"
+		  "widgets, and use their replacements."));
+      }
+
   command_buffer = g_strdup_printf ("glade-- %s", project->xml_filename);
   status = system (command_buffer);
   g_free (command_buffer);
@@ -482,7 +537,8 @@ glade_project_write_ada95_source (GladeProject *project)
     {
       return glade_error_new_general (GLADE_STATUS_ERROR,
 				      _("Error running gate to generate the Ada95 source code.\n"
-					"Check that you have gate installed and that it is in your PATH."));
+					"Check that you have gate installed and that it is in your PATH.\n"
+					"Then try running 'gate <project_file.glade>' in a terminal."));
     }
 
   return NULL;
@@ -504,7 +560,8 @@ glade_project_write_perl_source (GladeProject *project)
     {
       return glade_error_new_general (GLADE_STATUS_ERROR,
 				      _("Error running glade2perl to generate the Perl source code.\n"
-					"Check that you have glade2perl installed and that it is in your PATH."));
+					"Check that you have glade2perl installed and that it is in your PATH.\n"
+					"Then try running 'glade2perl <project_file.glade>' in a terminal."));
     }
 
   return NULL;
@@ -512,7 +569,7 @@ glade_project_write_perl_source (GladeProject *project)
 
 
 /* Use system() to run eglade on the XML file to generate Eiffel source code. */
-	static GladeError*
+static GladeError*
 glade_project_write_eiffel_source (GladeProject *project)
 {
 	gchar *command_buffer;
@@ -526,7 +583,8 @@ glade_project_write_eiffel_source (GladeProject *project)
 	{
 		return glade_error_new_general (GLADE_STATUS_ERROR,
 			_("Error running eglade to generate the Eiffel source code.\n"
-			"Check that you have eglade installed and that it is in your PATH."));
+			  "Check that you have eglade installed and that it is in your PATH.\n"
+			  "Then try running 'eglade <project_file.glade>' in a terminal."));
 	}
 
 	return NULL;
