@@ -1180,8 +1180,7 @@ glade_project_options_show_file_selection (GtkWidget *widget,
   filename = (char*) gtk_entry_get_text (GTK_ENTRY (options->directory_entry));
   if (glade_util_file_exists (filename))
     {
-      gtk_file_selection_set_filename (GTK_FILE_SELECTION (options->filesel),
-				       filename);
+      glade_util_set_file_selection_filename (options->filesel, filename);
     }
   if (GTK_IS_WINDOW (options))
     gtk_window_set_transient_for (GTK_WINDOW (options->filesel),
@@ -1198,8 +1197,9 @@ glade_project_options_set_filename (GtkWidget *widget,
 {
   gchar *filename;
 
-  filename = (char*) gtk_file_selection_get_filename (GTK_FILE_SELECTION (options->filesel));
+  filename = glade_util_get_file_selection_filename (options->filesel);
   gtk_entry_set_text (GTK_ENTRY (options->directory_entry), filename);
+  g_free (filename);
   glade_util_close_window (options->filesel);
 }
 
