@@ -55,6 +55,12 @@
  */
 static gchar *GladeProjectWindowKey = "GladeProjectWindowKey";
 
+/* The menuitem's to show/hide the various windows. */
+static GtkWidget *palette_item = NULL;
+static GtkWidget *property_editor_item = NULL;
+static GtkWidget *widget_tree_item = NULL;
+static GtkWidget *clipboard_item = NULL;
+
 
 static gboolean glade_project_window_delete_event (GtkWidget *widget,
 						   GdkEvent *event,
@@ -112,13 +118,13 @@ static void glade_project_window_delete (GtkWidget *widget,
 					 gpointer   data);
 static void glade_project_window_real_delete (GladeProjectWindow *project_window);
 
-static void glade_project_window_show_palette (GtkWidget *widget,
+static void glade_project_window_toggle_palette (GtkWidget *widget,
 					       gpointer   data);
-static void glade_project_window_show_property_editor (GtkWidget *widget,
+static void glade_project_window_toggle_property_editor (GtkWidget *widget,
 						       gpointer   data);
-static void glade_project_window_show_widget_tree (GtkWidget *widget,
+static void glade_project_window_toggle_widget_tree (GtkWidget *widget,
 						   gpointer   data);
-static void glade_project_window_show_clipboard (GtkWidget *widget,
+static void glade_project_window_toggle_clipboard (GtkWidget *widget,
 						 gpointer   data);
 static void glade_project_window_toggle_tooltips (GtkWidget *widget,
 						  gpointer   data);
@@ -224,32 +230,40 @@ static GnomeUIInfo EditMenu[] =
   GNOMEUIINFO_END
 };
 
+
+/* These must match the indices of the appropriate items in the settings
+   GnomeUIInfo array. We need them to set the initial state. */
+#define GLADE_PALETTE_ITEM 0
+#define GLADE_PROPERTY_EDITOR_ITEM 1
+#define GLADE_WIDGET_TREE_ITEM 2
+#define GLADE_CLIPBOARD_ITEM 3
+
 static GnomeUIInfo ViewMenu[] =
 {
   {
-    GNOME_APP_UI_ITEM, N_("Show _Palette"), N_("Show the palette of widgets"),
-    glade_project_window_show_palette, NULL, NULL,
+    GNOME_APP_UI_TOGGLEITEM, N_("Show _Palette"), N_("Show the palette of widgets"),
+    glade_project_window_toggle_palette, NULL, NULL,
     GNOME_APP_PIXMAP_NONE, NULL,
     0, 0, NULL
   },
   {
-    GNOME_APP_UI_ITEM, N_("Show Property _Editor"),
+    GNOME_APP_UI_TOGGLEITEM, N_("Show Property _Editor"),
     N_("Show the property editor"),
-    glade_project_window_show_property_editor, NULL, NULL,
+    glade_project_window_toggle_property_editor, NULL, NULL,
     GNOME_APP_PIXMAP_NONE, NULL,
     0, 0, NULL
   },
   {
-    GNOME_APP_UI_ITEM, N_("Show Widget _Tree"),
+    GNOME_APP_UI_TOGGLEITEM, N_("Show Widget _Tree"),
     N_("Show the widget tree"),
-    glade_project_window_show_widget_tree, NULL, NULL,
+    glade_project_window_toggle_widget_tree, NULL, NULL,
     GNOME_APP_PIXMAP_NONE, NULL,
     0, 0, NULL
   },
   {
-    GNOME_APP_UI_ITEM, N_("Show _Clipboard"),
+    GNOME_APP_UI_TOGGLEITEM, N_("Show _Clipboard"),
     N_("Show the clipboard"),
-    glade_project_window_show_clipboard, NULL, NULL,
+    glade_project_window_toggle_clipboard, NULL, NULL,
     GNOME_APP_PIXMAP_NONE, NULL,
     0, 0, NULL
   },
@@ -414,7 +428,24 @@ glade_project_window_new (void)
 			    NULL);
 
   gnome_app_create_menus (GNOME_APP (project_window->window), MainMenu);
+
   /* Set the states of the toggle items. */
+  palette_item = ViewMenu[GLADE_PALETTE_ITEM].widget;
+  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (palette_item),
+				  TRUE);
+
+  property_editor_item = ViewMenu[GLADE_PROPERTY_EDITOR_ITEM].widget;
+  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (property_editor_item),
+				  TRUE);
+
+  widget_tree_item = ViewMenu[GLADE_WIDGET_TREE_ITEM].widget;
+  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (widget_tree_item),
+				  FALSE);
+
+  clipboard_item = ViewMenu[GLADE_CLIPBOARD_ITEM].widget;
+  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (clipboard_item),
+				  FALSE);
+
   show_grid_item = SettingsMenu[GLADE_SHOW_GRID_ITEM].widget;
   gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (show_grid_item),
 				 editor_get_show_grid ());
@@ -689,43 +720,59 @@ glade_project_window_new (void)
   menu = gtk_menu_new ();
   gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), menu);
 
-  menuitem = gtk_menu_item_new_with_mnemonic (_("Show _Palette"));
+  menuitem = gtk_check_menu_item_new_with_mnemonic (_("Show _Palette"));
   gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
-		      GTK_SIGNAL_FUNC (glade_project_window_show_palette),
+		      GTK_SIGNAL_FUNC (glade_project_window_toggle_palette),
 		      NULL);
   gtk_container_add (GTK_CONTAINER (menu), menuitem);
   gtk_tooltips_set_tip (tooltips, menuitem, _("Show the palette of widgets"),
 			NULL);
+  palette_item = menuitem;
+  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (palette_item),
+				  TRUE);
   gtk_widget_show (menuitem);
 
-  menuitem = gtk_menu_item_new_with_mnemonic (_("Show P_roperties"));
+  menuitem = gtk_check_menu_item_new_with_mnemonic (_("Show Property _Editor"));
   gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
-		      GTK_SIGNAL_FUNC (glade_project_window_show_property_editor),
+		      GTK_SIGNAL_FUNC (glade_project_window_toggle_property_editor),
 		      NULL);
   gtk_container_add (GTK_CONTAINER (menu), menuitem);
   gtk_tooltips_set_tip (tooltips, menuitem,
-			_("Show the properties of the selected widget"), NULL);
+			_("Show the property editor"), NULL);
+  property_editor_item = menuitem;
+  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (property_editor_item),
+				  TRUE);
   gtk_widget_show (menuitem);
 
-  menuitem = gtk_menu_item_new_with_mnemonic (_("Show _Widget Tree"));
+  menuitem = gtk_check_menu_item_new_with_mnemonic (_("Show Widget _Tree"));
   gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
-		      GTK_SIGNAL_FUNC (glade_project_window_show_widget_tree),
+		      GTK_SIGNAL_FUNC (glade_project_window_toggle_widget_tree),
 		      NULL);
   gtk_container_add (GTK_CONTAINER (menu), menuitem);
   gtk_tooltips_set_tip (tooltips, menuitem,
-			_("Show the project widget tree"), NULL);
+			_("Show the widget tree"), NULL);
+  widget_tree_item = menuitem;
+  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (widget_tree_item),
+				  FALSE);
   gtk_widget_show (menuitem);
 
-  menuitem = gtk_menu_item_new_with_mnemonic (_("Show _Clipboard"));
+  menuitem = gtk_check_menu_item_new_with_mnemonic (_("Show _Clipboard"));
   gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
-		      GTK_SIGNAL_FUNC (glade_project_window_show_clipboard),
+		      GTK_SIGNAL_FUNC (glade_project_window_toggle_clipboard),
 		      NULL);
   gtk_container_add (GTK_CONTAINER (menu), menuitem);
   gtk_tooltips_set_tip (tooltips, menuitem,
 			_("Show the clipboard"), NULL);
+  clipboard_item = menuitem;
+  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (clipboard_item),
+				  FALSE);
   gtk_widget_show (menuitem);
 
-  menuitem = gtk_check_menu_item_new_with_mnemonic (_("Show Widget _Tooltips"));
+  menuitem = gtk_separator_menu_item_new ();
+  gtk_container_add (GTK_CONTAINER (menu), menuitem);
+  gtk_widget_show (menuitem);
+
+  menuitem = gtk_check_menu_item_new_with_mnemonic (_("Show _Widget Tooltips"));
   gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menuitem),
 				 gb_widget_get_show_tooltips ());
   gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
@@ -964,6 +1011,31 @@ glade_project_window_new_project (GtkWidget *widget,
 }
 
 
+/* This sets the palette sensitive if a project is open or insensitive if
+   not, and shows/hides the GNOME widgets as appropriate. */
+static void
+glade_project_window_setup_interface (GladeProjectWindow *project_window)
+{
+  gboolean gnome_support = TRUE;
+  gboolean gnome_db_support = TRUE;
+
+  /* Make sure the arrow is selected in the palette. */
+  glade_palette_reset_selection (GLADE_PALETTE (glade_palette), FALSE);
+
+  if (current_project)
+    {
+      gnome_support = glade_project_get_gnome_support (current_project);
+      gnome_db_support = glade_project_get_gnome_db_support (current_project);
+    }
+
+  glade_palette_set_show_gnome_widgets (GLADE_PALETTE (glade_palette),
+					gnome_support, gnome_db_support);
+
+  gtk_widget_set_sensitive (GTK_BIN (glade_palette)->child,
+			    current_project ? TRUE : FALSE);
+}
+
+
 static void
 glade_project_window_on_new_project_ok (GladeProjectWindow *project_window)
 {
@@ -1022,8 +1094,9 @@ glade_project_window_on_new_project_ok (GladeProjectWindow *project_window)
   gtk_statusbar_push (GTK_STATUSBAR (project_window->statusbar), 1,
 		      _("New project created."));
 
-  /* Make sure the palette is sensitive now. */
-  gtk_widget_set_sensitive (GTK_BIN (glade_palette)->child, TRUE);
+  /* Make sure the palette is sensitive now and shows the appropriate
+     widgets. */
+  glade_project_window_setup_interface (project_window);
 }
 
 
@@ -1044,8 +1117,7 @@ glade_project_window_on_open_project (GtkWidget *widget,
     gtk_window_set_transient_for (GTK_WINDOW (filesel),
 				  GTK_WINDOW (project_window->window));
   if (project_window->current_directory)
-    gtk_file_selection_set_filename (GTK_FILE_SELECTION (filesel),
-				     project_window->current_directory);
+    glade_util_set_file_selection_filename (filesel, project_window->current_directory);
 
   gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filesel)->ok_button),
 		      "clicked",
@@ -1069,9 +1141,9 @@ glade_project_window_real_open_project (GtkWidget          *widget,
   gchar *filename;
 
   filesel = gtk_widget_get_toplevel (widget);
-  filename = (char*) gtk_file_selection_get_filename (GTK_FILE_SELECTION (filesel));
-
+  filename = glade_util_get_file_selection_filename (filesel);
   glade_project_window_open_project (project_window, filename);
+  g_free (filename);
   gtk_widget_destroy (filesel);
 }
 
@@ -1097,8 +1169,6 @@ glade_project_window_open_project (GladeProjectWindow *project_window,
       glade_project_view_set_project (GLADE_PROJECT_VIEW (project_window->project_view), project);
       gtk_statusbar_push (GTK_STATUSBAR (project_window->statusbar), 1,
 			  _("Project opened."));
-      /* Make sure the palette is sensitive now. */
-      gtk_widget_set_sensitive (GTK_BIN (glade_palette)->child, TRUE);
     }
   else
     {
@@ -1107,7 +1177,6 @@ glade_project_window_open_project (GladeProjectWindow *project_window,
 	 create a new empty GTK+ project. */
 #ifdef USE_GNOME
       project = NULL;
-      gtk_widget_set_sensitive (GTK_BIN (glade_palette)->child, FALSE);
 #else
       project = glade_project_new ();
 #endif
@@ -1115,6 +1184,11 @@ glade_project_window_open_project (GladeProjectWindow *project_window,
       gtk_statusbar_push (GTK_STATUSBAR (project_window->statusbar), 1,
 			  _("Error opening project."));
     }
+
+  /* Make sure the palette is sensitive if needed and show the appropriate
+     widgets. */
+  glade_project_window_setup_interface (project_window);
+
   glade_project_window_update_title (project_window);
 }
 
@@ -1279,6 +1353,7 @@ glade_project_window_options_ok (GtkWidget	    *widget,
   action = options->action;
 
   glade_project_window_update_title (project_window);
+  glade_project_window_setup_interface (project_window);
 
   gtk_widget_destroy (GTK_WIDGET (options));
 
@@ -1356,8 +1431,7 @@ glade_project_window_save_project_as (GladeProjectWindow *project_window)
     gtk_window_set_transient_for (GTK_WINDOW (filesel),
 				  GTK_WINDOW (project_window->window));
   if (project_window->current_directory)
-    gtk_file_selection_set_filename (GTK_FILE_SELECTION (filesel),
-				     project_window->current_directory);
+    glade_util_set_file_selection_filename (filesel, project_window->current_directory);
   gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filesel)->ok_button),
 		      "clicked",
 		      GTK_SIGNAL_FUNC (glade_project_window_save_as_callback),
@@ -1382,7 +1456,7 @@ glade_project_window_save_as_callback (GtkWidget          *widget,
   gboolean saved;
 
   filesel = gtk_widget_get_toplevel (widget);
-  filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (filesel));
+  filename = glade_util_get_file_selection_filename (filesel);
 
   g_free (project_window->current_directory);
   project_window->current_directory = glade_util_dirname (filename);
@@ -1394,6 +1468,7 @@ glade_project_window_save_as_callback (GtkWidget          *widget,
       glade_project_window_real_save_project (project_window, TRUE);
     }
   gtk_widget_destroy (filesel);
+  g_free (filename);
 }
 #endif
 
@@ -1669,35 +1744,91 @@ glade_project_window_real_delete (GladeProjectWindow *project_window)
 }
 
 
-static void
-glade_project_window_show_palette (GtkWidget *widget,
-				   gpointer   data)
+void
+glade_project_window_uncheck_palette_menu_item (void)
 {
-  glade_show_palette ();
+  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (palette_item),
+				  FALSE);
+}
+
+
+void
+glade_project_window_uncheck_property_editor_menu_item (void)
+{
+  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (property_editor_item),
+				  FALSE);
+}
+
+
+void
+glade_project_window_uncheck_widget_tree_menu_item (void)
+{
+  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (widget_tree_item),
+				  FALSE);
+}
+
+
+void
+glade_project_window_uncheck_clipboard_menu_item (void)
+{
+  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (clipboard_item),
+				  FALSE);
 }
 
 
 static void
-glade_project_window_show_property_editor (GtkWidget *widget,
+glade_project_window_toggle_palette (GtkWidget *widget,
+				      gpointer   data)
+{
+  gboolean show_palette;
+
+  show_palette = GTK_CHECK_MENU_ITEM (widget)->active;
+  if (show_palette)
+    glade_show_palette ();
+  else
+    glade_hide_palette ();
+}
+
+
+static void
+glade_project_window_toggle_property_editor (GtkWidget *widget,
 					   gpointer   data)
 {
-  glade_show_property_editor ();
+  gboolean show_property_editor;
+
+  show_property_editor = GTK_CHECK_MENU_ITEM (widget)->active;
+  if (show_property_editor)
+    glade_show_property_editor ();
+  else
+    glade_hide_property_editor ();
 }
 
 
 static void
-glade_project_window_show_widget_tree (GtkWidget *widget,
+glade_project_window_toggle_widget_tree (GtkWidget *widget,
 				       gpointer   data)
 {
-  glade_show_widget_tree ();
+  gboolean show_widget_tree;
+
+  show_widget_tree =  GTK_CHECK_MENU_ITEM (widget)->active;
+  if (show_widget_tree)
+    glade_show_widget_tree ();
+  else
+    glade_hide_widget_tree ();
 }
 
 
 static void
-glade_project_window_show_clipboard (GtkWidget *widget,
+glade_project_window_toggle_clipboard (GtkWidget *widget,
 				     gpointer   data)
 {
-  glade_show_clipboard ();
+  gboolean show_clipboard;
+
+  show_clipboard =  GTK_CHECK_MENU_ITEM (widget)->active;
+  if (show_clipboard)
+    glade_show_clipboard ();
+  else
+    glade_hide_clipboard ();
 }
 
 
