@@ -21,6 +21,7 @@
 #include <gnome.h>
 #include "../gb.h"
 #include "../tree.h"
+#include "../glade_gnome.h"
 
 /* Include the 21x21 icon pixmap for this widget, to be used in the palette */
 #include "../graphics/gnome-app.xpm"
@@ -58,7 +59,6 @@ static gchar *StatusBar = "GnomeApp::status_bar";
 
 
 static void gb_gnome_app_setup_initial_app (GtkWidget *widget);
-static void gb_gnome_app_setup_menu_items (GnomeUIInfo *uiinfo, gint *indices);
 
 /******
  * NOTE: To use these functions you need to uncomment them AND add a pointer
