@@ -28,6 +28,7 @@
 #include "palette.h"
 #include "glade_palette.h"
 #include "glade_project.h"
+#include "glade_project_window.h"
 
 
 static void palette_create (void);
@@ -51,6 +52,10 @@ palette_create ()
   gtk_signal_connect (GTK_OBJECT (glade_palette), "delete_event",
 		      GTK_SIGNAL_FUNC (palette_hide), NULL);
 
+  gtk_signal_connect (GTK_OBJECT (glade_palette), "hide",
+                      GTK_SIGNAL_FUNC (glade_project_window_uncheck_palette_menu_item),
+                      NULL);
+
   gtk_window_set_title (GTK_WINDOW (glade_palette), _("Palette"));
   gtk_container_set_border_width (GTK_CONTAINER (glade_palette), 0);
   colormap = gtk_widget_get_colormap (glade_palette);
