@@ -168,7 +168,7 @@ save_project_file_internal (GladeProject *project)
 	goto out;
     }
 
-  fp = fopen (filename, "w");
+  fp = glade_util_fopen (filename, "w");
   if (fp == NULL)
     {
       error = glade_error_new_system (_("Couldn't create file:\n  %s\n"),
@@ -220,7 +220,7 @@ save_xml_file_internal (GladeProject *project)
 	return data.error;
     }
 
-  fp = fopen (filename, "w");
+  fp = glade_util_fopen (filename, "w");
   if (fp == NULL)
     return glade_error_new_system (_("Couldn't create file:\n  %s\n"),
 				   filename);
@@ -335,8 +335,11 @@ save_requires_tags (FILE *fp, GladeProject *project)
 				   (GtkCallback) save_requires_tags_cb,
 				   &requires_data);
 
-  /* Require "gnome", as provided in libgnomeui/glade/glade-gnome.c. */
-  if (requires_data.require_gnome)
+  /* Require "gnome", as provided in libgnomeui/glade/glade-gnome.c.
+     We now require GNOME for all GNOME projects, since even if they don't
+     contain GNOME widgets they may use stock GNOME items. */
+  if (requires_data.require_gnome
+      || glade_project_get_gnome_support (project))
     fprintf (fp, "<requires lib=\"gnome\"/>\n");
   /* Require "canvas", as provided in libgnomecanvas/glade/glade-canvas.c. */
   if (requires_data.require_gnome_canvas)
@@ -909,7 +912,7 @@ save_translatable_strings (GbWidgetGetArgData * data)
 
   filename = glade_project_get_translatable_strings_file (data->project);
 
-  fp = fopen (filename, "w");
+  fp = glade_util_fopen (filename, "w");
   if (fp == NULL)
     {
       data->error = glade_error_new_system (_("Couldn't create file:\n  %s\n"),
