@@ -1138,14 +1138,18 @@ glade_util_close_window (GtkWidget * widget)
 }
 
 
-/* Returns TRUE if the given file exists. */
+/* Returns TRUE if the given file exists. filename must be UTF-8. */
 gboolean
 glade_util_file_exists (const gchar *filename)
 {
   int status;
   struct stat filestat;
+  gchar *on_disk_filename;
+
+  on_disk_filename = g_filename_from_utf8 (filename, -1, NULL, NULL, NULL);
+  status = stat (on_disk_filename, &filestat);
+  g_free (on_disk_filename);
 
-  status = stat (filename, &filestat);
   if (status == -1 && errno == ENOENT)
     return FALSE;
   return TRUE;
@@ -1153,14 +1157,18 @@ glade_util_file_exists (const gchar *filename)
 
 
 /* Returns the last modification time of the given file, or 0 if it doesn't
-   exist, or -1 on error. */
+   exist, or -1 on error. filename must be UTF-8. */
 GladeError*
 glade_util_file_last_mod_time (const gchar *filename, time_t *last_mod_time)
 {
   int status;
   struct stat filestat;
+  gchar *on_disk_filename;
+
+  on_disk_filename = g_filename_from_utf8 (filename, -1, NULL, NULL, NULL);
+  status = stat (on_disk_filename, &filestat);
+  g_free (on_disk_filename);
 
-  status = stat (filename, &filestat);
   if (status == -1)
     {
       return glade_error_new_system (_("Couldn't access file:\n  %s\n"),
@@ -1173,7 +1181,7 @@ glade_util_file_last_mod_time (const gchar *filename, time_t *last_mod_time)
 
 
 /* This copies a file from src to dest, and returns a GladeError if an error
-   occurs. */
+   occurs. src & dest must be UTF-8. */
 GladeError*
 glade_util_copy_file (const gchar *src,
 		      const gchar *dest)
@@ -1183,13 +1191,13 @@ glade_util_copy_file (const gchar *src,
   gint bytes_read, bytes_written;
   GladeError *error = NULL;
 
-  input_fp = fopen (src, "r");
+  input_fp = glade_util_fopen (src, "r");
   if (input_fp == NULL)
     {
       return glade_error_new_system (_("Couldn't open file:\n  %s\n"), src);
     }
 
-  output_fp = fopen (dest, "w");
+  output_fp = glade_util_fopen (dest, "w");
   if (output_fp == NULL)
     {
       error = glade_error_new_system (_("Couldn't create file:\n  %s\n"),
@@ -1233,20 +1241,22 @@ glade_util_copy_file (const gchar *src,
 
 
 /* Creates a directory if it doesn't already exist. directory must be an
-   absolute path. */
+   absolute path, in UTF-8. */
 GladeError*
 glade_util_ensure_directory_exists (const gchar *directory)
 {
+  GladeError *error = NULL;
   struct stat filestat;
+  gchar *on_disk_directory;
 
   g_return_val_if_fail (g_path_is_absolute (directory), NULL);
 
-  if (stat (directory, &filestat) != 0)
+  on_disk_directory = g_filename_from_utf8 (directory, -1, NULL, NULL, NULL);
+  if (stat (on_disk_directory, &filestat) != 0)
     {
       /* If the directory doesn't exist, try to create it. */
       if (errno == ENOENT)
 	{
-	  GladeError *error;
 	  gchar *parent_dir;
 
 	  /* First make sure the parent directory exists. */
@@ -1255,39 +1265,42 @@ glade_util_ensure_directory_exists (const gchar *directory)
 	    {
 	      error = glade_util_ensure_directory_exists (parent_dir);
 	      g_free (parent_dir);
-	      if (error)
-		return error;
 	    }
 
-#ifdef _WIN32
-	  if (mkdir (directory) != 0)
-#else
-	  if (mkdir (directory, 0777) != 0)
-#endif
+	  if (!error)
 	    {
+#ifdef _WIN32
+	      if (mkdir (on_disk_directory) != 0)
+#else
+	      if (mkdir (on_disk_directory, 0777) != 0)
+#endif
+		{
 #ifndef _WIN32
 /* This happens under WIN32 when stat is confused by the filename, but this is
    harmless, since we know that the directory exists after all. */
-	      return glade_error_new_system (_("Couldn't create directory:\n  %s\n"), directory);
+		  error = glade_error_new_system (_("Couldn't create directory:\n  %s\n"), directory);
 #endif
+		}
 	    }
 	}
       else
 	{
-	  return glade_error_new_system (_("Couldn't access directory:\n  %s\n"), directory);
+	  error = glade_error_new_system (_("Couldn't access directory:\n  %s\n"), directory);
 	}
     }
 #ifndef _WIN32
   /* If the directory does exist, check it is a directory. */
   else if (!S_ISDIR (filestat.st_mode))
     {
-      return glade_error_new_general (GLADE_STATUS_INVALID_DIRECTORY,
-				      _("Invalid directory:\n  %s\n"),
-				      directory);
+      error = glade_error_new_general (GLADE_STATUS_INVALID_DIRECTORY,
+				       _("Invalid directory:\n  %s\n"),
+				       directory);
     }
 #endif
 
-  return NULL;
+  g_free (on_disk_directory);
+
+  return error;
 }
 
 
@@ -2143,3 +2156,48 @@ glade_util_uses_border_width (GtkWidget *widget)
 
   return FALSE;
 }
+
+
+/* Converts a filename from UTF-8 to on-disk encoding, and sets it in a
+   GtkFileSelection. */
+void
+glade_util_set_file_selection_filename (GtkWidget *filesel,
+					const gchar *filename_utf8)
+{
+  gchar *on_disk_filename;
+  
+  on_disk_filename = g_filename_from_utf8 (filename_utf8, -1, NULL, NULL,
+					   NULL);
+  gtk_file_selection_set_filename (GTK_FILE_SELECTION (filesel),
+				   on_disk_filename);
+  g_free (on_disk_filename);
+}
+
+
+/* Gets the selected file in a GtkFileSelection, converts it to UTF-8 and
+   returns it. Note that the returned string must be freed. */
+gchar*
+glade_util_get_file_selection_filename (GtkWidget *filesel)
+{
+  const gchar *on_disk_filename;
+  gchar *filename;
+  
+  on_disk_filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (filesel));
+  filename = g_filename_to_utf8 (on_disk_filename, -1, NULL, NULL, NULL);
+  return filename;
+}
+
+
+/* Like fopen but takes a UTF-8 filename and converts to on-disk encoding. */
+FILE*
+glade_util_fopen (const gchar *filename_utf8, const gchar *mode)
+{
+  gchar *on_disk_filename;
+  FILE *fp;
+
+  on_disk_filename = g_filename_from_utf8 (filename_utf8, -1, NULL, NULL,
+					   NULL);
+  fp = fopen (on_disk_filename, mode);
+  g_free (on_disk_filename);
+  return fp;
+}
