@@ -18,6 +18,8 @@
 #ifndef GLADE_UTILS_H
 #define GLADE_UTILS_H
 
+#include <stdio.h>
+
 #include <gtk/gtkbox.h>
 #include <gtk/gtkcombo.h>
 #include <gtk/gtkfixed.h>
@@ -249,13 +251,16 @@ gint	  glade_util_close_window		(GtkWidget	*widget);
  * File Utility Functions.
  */
 
-/* Returns TRUE if the given file exists. */
+/* Returns TRUE if the given file exists. filename must be UTF-8. */
 gboolean    glade_util_file_exists		(const gchar *filename);
 
+/* Returns the last modification time of the given file, or 0 if it doesn't
+   exist, or -1 on error. filename must be UTF-8. */
 GladeError* glade_util_file_last_mod_time	(const gchar *filename,
 						 time_t	     *last_mod_time);
 
-/* Copies src to dest, returning a GladeError if an error occurs. */
+/* This copies a file from src to dest, and returns a GladeError if an error
+   occurs. src & dest must be UTF-8. */
 GladeError* glade_util_copy_file		(const gchar *src,
 						 const gchar *dest);
 
@@ -342,6 +347,18 @@ gchar	 *glade_util_gtk_combo_func		(gpointer	 listitem);
 /* Returns TRUE if we need to support a 'Border Width' property for a widget.*/
 gboolean  glade_util_uses_border_width		(GtkWidget	*widget);
 
+/* Converts a filename from UTF-8 to on-disk encoding, and sets it in a
+   GtkFileSelection. */
+void	  glade_util_set_file_selection_filename(GtkWidget	*filesel,
+						 const gchar	*filename_utf8);
+/* Gets the selected file in a GtkFileSelection, converts it to UTF-8 and
+   returns it. Note that the returned string must be freed. */
+gchar*	  glade_util_get_file_selection_filename(GtkWidget *filesel);
+
+/* Like fopen but takes a UTF-8 filename and converts to on-disk encoding. */
+FILE*	  glade_util_fopen			(const gchar	*filename_utf8,
+						 const gchar	*mode);
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
